package com.client;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.ext.xml.DomRepresentation;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.ClientResource;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.IOException;
import java.sql.Date;
import java.util.List;

import static org.junit.Assert.*;

public class UserPuzzleTest {
    private FileHelper helper;
    private ClientResource userClient;
    private ClientResource puzzleClient;
    private ClientResource puzzlesClient;

    @Before
    public void init() {
        helper = new FileHelper();
        userClient = new ClientResource("http://localhost:8080/user/");
        puzzleClient = new ClientResource("http://localhost:8080/puzzle/");
        puzzlesClient = new ClientResource("http://localhost:8080/puzzles/");
    }

    @Test
    public void getPuzzles() throws IOException, SAXException, ParserConfigurationException {

        DomRepresentation empty = new DomRepresentation(puzzlesClient.get());
        assertNotNull(empty);

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        File puzzleFile2 = helper.getPuzzleFile2();
        File puzzleFile3 = helper.getPuzzleFile3();

        // Parsing the XML puzzle file to create a DomRepresentation to send in the put method
        addUser1();
        String user2Id = addUser2().getText();
        Document document2 = builder.parse(puzzleFile2);
        Document document3 = builder.parse(puzzleFile3);
        DomRepresentation rep2 = new DomRepresentation(MediaType.TEXT_XML, document2);
        DomRepresentation rep3 = new DomRepresentation(MediaType.TEXT_XML, document3);
        puzzleClient.put(rep2);
        puzzleClient.put(rep3);
        addPuzzle();

        DomRepresentation rep = new DomRepresentation(puzzlesClient.get());
        List<Node> puzzles = rep.getNodes("/puzzles/puzzle");
        // Number of puzzles returned
        assertEquals(2, puzzles.size());

        // Get all puzzles which user2 can edit
        puzzlesClient = new ClientResource("http://localhost:8080/puzzles/" + user2Id + "?editable");
        rep = new DomRepresentation(puzzlesClient.get());
        puzzles = rep.getNodes("/puzzles/puzzle");
        // Number of puzzles returned
        assertEquals(1, puzzles.size());

        // Get all puzzles with Math category which is published
        puzzlesClient = new ClientResource("http://localhost:8080/puzzles/Math");
        rep = new DomRepresentation(puzzlesClient.get());
        puzzles = rep.getNodes("/puzzles/puzzle");
        assertEquals(2, puzzles.size());

        // Get all puzzles with Physics category which is published
        puzzlesClient = new ClientResource("http://localhost:8080/puzzles/Physics");
        rep = new DomRepresentation(puzzlesClient.get());
        puzzles = rep.getNodes("/puzzles/puzzle");
        assertEquals(0, puzzles.size());
    }
    @Test
    public void putGetDeletePuzzle() throws ParserConfigurationException, IOException, SAXException {
        addUser1();
        addUser2();
        // Adding puzzle
        Representation puzzleId = addPuzzle();
        String id = puzzleId.getText();

        // If put returns 0 the puzzle has not been added
        assertNotSame("Put method returned null", null, id);
        assertNotSame("Put method returned 0", "0", id);

        // Getting the previously added puzzle by using the id which the put method returned
        // Puzzle must also have the id as content
        puzzleClient = new ClientResource(Method.GET, "http://localhost:8080/puzzle/" + id);
        DomRepresentation puzzle = new DomRepresentation(puzzleClient.get());
        String testId = puzzle.getText("/puzzle/id");
        assertEquals("Lookup id not the same as xml stored id", id, testId);

        // Number of rows affected
        assertEquals("Number of rows affected not 1","1", puzzleClient.delete().getText());

        // The user is indeed deleted and get returns null(an empty DomRepresentation)
        //assertNull("Puzzle.get not null", puzzleClient.get().getText());
    }

    @Test
    public void putGetDeleteUser() throws IOException, SAXException, ParserConfigurationException {
        Representation userId = addUser1();
        String id = userId.getText();

        // If put returns 0 the user has not been added
        assertNotSame("Put method returned null", null, id);
        assertNotSame("Put method returned 0", "0", id);

        // Getting the previously added user by using the userId which the put method returned
        userClient = new ClientResource(Method.GET, "http://localhost:8080/user/" + id);
        DomRepresentation user = new DomRepresentation(userClient.get());
        assertEquals(id, user.getText("/user/id"));

        // Number of rows affected
        assertEquals("1", userClient.delete().getText());

        // The user is indeed deleted and get returns null(an empty DomRepresentation)
        assertNull(userClient.get().getText());
    }

    @Test
    public void publishPuzzle() throws ParserConfigurationException, IOException, SAXException {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        File puzzleFile = helper.getPuzzleFile3();
        // Parsing the XML puzzle file to create a DomRepresentation to send in the put method
        Document document = builder.parse(puzzleFile);
        DomRepresentation rep = new DomRepresentation(MediaType.TEXT_XML, document);
        // Adding user which has written the puzzle, puzzle's author
        addUser1();
        addUser2();

        Representation puzzle = puzzleClient.put(rep);
        String id = puzzle.getText();

        ClientResource publishClient = new ClientResource(Method.PUT, "http://localhost:8080/publish/" + id);
        Representation publish = publishClient.put(rep);
        String publishValue = publish.getText();
        assertEquals("Number of rows affected is not 1", 1, Integer.parseInt(publishValue));
    }

    @Test
    public void highscoreTest() throws ParserConfigurationException, IOException, SAXException {
        Representation user = addUser1();
        String userId = user.getText();
        addUser2();
        Representation puzzle = addPuzzle();
        String puzzleId = puzzle.getText();
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        File highscoreFile = helper.getAddHighscoreFile();

        Document document = builder.parse(highscoreFile);
        DomRepresentation rep = new DomRepresentation(MediaType.TEXT_XML, document);

        ClientResource highscoreClient = new ClientResource("http://localhost:8080/highscore/" + puzzleId + "," + userId);
        Representation highscoreUpdate = highscoreClient.put(rep);
        String rowsUpdated = highscoreUpdate.getText();

        assertEquals("Highscore not updated", "1", rowsUpdated);

        highscoreClient = new ClientResource("http://localhost:8080/highscore/" + puzzleId);
        DomRepresentation highscore = new DomRepresentation(highscoreClient.get());

        List<Node> highscores = highscore.getNodes("/scores/score");

        assertEquals("No highscore received", 1, highscores.size());
    }

    @Test
    public void addFriend() throws IOException, SAXException, ParserConfigurationException {
        Representation user = addUser1();
        String userId = user.getText();

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        File userFile = helper.getUserFile2();

        // Parsing the XML user file to create a DomRepresentation to send in the put method
        Document document = builder.parse(userFile);
        DomRepresentation rep = new DomRepresentation(MediaType.TEXT_XML, document);
        Representation friend = userClient.put(rep);
        String friendId = friend.getText();
        // UserId is to get a new friend
        ClientResource friendClient = new ClientResource(Method.PUT, "http://localhost:8080/friend/" + userId);
        // Save user to be added as a friend
        File addFriendFile = helper.getNewFriendFile();
        document = builder.parse(addFriendFile);
        DomRepresentation newFriend = new DomRepresentation(MediaType.TEXT_XML, document);

        // Add a friend by xml with the friend's email
        Representation friendResponse = friendClient.put(newFriend);

        assertEquals("Friend not added", "1", friendResponse.getText());

        DomRepresentation friends = new DomRepresentation(friendClient.get());

        List<Node> friendlist = friends.getNodes("/friends/friend");
        assertEquals("Friendlist not received", 1, friendlist.size());
    }

    /**
     * PUT router.attach("/game/{requestId},{winnerId},{winnerScore}", GameServerResource.class); URI for updating a request
     * GET and DELETE router.attach("/game/{id}", GameServerResource.class);  URI for getRequests and deleteRequest  "?challenger"
     * PUT router.attach("/game/", GameServerResource.class);     URI for add and update request
     */
    @Test
    public void gameRequest() throws ParserConfigurationException, IOException, SAXException {
        ClientResource requestClient = new ClientResource("http://localhost:8080/game/");

        // Initialize by adding users and puzzle
        Representation challenger = addUser1();
        Representation receiver = addUser2();
        String receiverId = receiver.getText();
        String challengerId = challenger.getText();
        Representation puzzle = addPuzzle();
        String puzzleId = puzzle.getText();

        // Generate request representation
        DomRepresentation request = new DomRepresentation(MediaType.TEXT_XML);
        Document doc = request.getDocument();
        generateRequestNode(receiverId,challengerId,puzzleId,doc);

        // First we add a request as a challenger
        Representation rep = requestClient.put(request);
        String requestId = rep.getText();
        assertNotNull("Request not added", requestId);

        // Then we get the request as the receiver
        requestClient = new ClientResource("http://localhost:8080/game/" + receiverId);
        DomRepresentation receiverRequest = new DomRepresentation(requestClient.get());
        String newRequestId = receiverRequest.getText("/gameRequests/gameRequest/requestId");
        assertEquals("RequestId not correct", requestId, newRequestId);

        // Then we return the completed request from the receiver
        requestClient = new ClientResource("http://localhost:8080/game/" + newRequestId + "," + challengerId + "," + "500");
        Representation updateResponse = requestClient.put(null);
        String responseText = updateResponse.getText();
        assertEquals("Number of rows updated not 1", "1", responseText);

        // Then we get the request again as the challenger
        requestClient = new ClientResource("http://localhost:8080/game/" + challengerId + "?challenger");
        DomRepresentation challengerResponse = new DomRepresentation(requestClient.get());
        String challengerRequestId = challengerResponse.getText("/gameRequests/gameRequest/requestId");
        assertEquals("RequestId received by challenger not correct", requestId, challengerRequestId);

        // Then we delete the request because it has been seen
        requestClient = new ClientResource("http://localhost:8080/game/" + requestId);
        Representation delete = requestClient.delete();
        String rowsDeleted = delete.getText();
        assertEquals("Number of rows deletet not 1", "1", rowsDeleted);
    }

    @Test
    public void testRating() throws ParserConfigurationException, SAXException, IOException {
        addUser1();
        addUser2();
        Representation puzzle = addPuzzle();
        String puzzleId = puzzle.getText();
        puzzleClient = new ClientResource("http://localhost:8080/puzzle/" + puzzleId + ",7?rating");

        String numberOfRowsAffected = puzzleClient.put(new StringRepresentation("")).getText();
        assertEquals("Number of rows not 1", "1", numberOfRowsAffected);

        DomRepresentation storedPuzzle = new DomRepresentation(puzzleClient.get());
        String puzzleRating = storedPuzzle.getText("/puzzle/rating");
        assertEquals("Rating not correct", "7", puzzleRating);

        puzzleClient = new ClientResource("http://localhost:8080/puzzle/" + puzzleId + ",1?rating");
        puzzleClient.put(new StringRepresentation(""));

        storedPuzzle = new DomRepresentation(puzzleClient.get());
        puzzleRating = storedPuzzle.getText("/puzzle/rating");
        assertEquals("Second rating not correct", "4", puzzleRating);
    }

    private void generateRequestNode(String receiverId, String challengerId, String puzzleId, Document doc){
        Node requestElt = doc.createElement("gameRequest");
        doc.appendChild(requestElt);

        Node requestIdElt = doc.createElement("requestId");
        requestIdElt.setTextContent("");
        requestElt.appendChild(requestIdElt);

        Node challengerElt = doc.createElement("challengerId");
        challengerElt.setTextContent(String.valueOf(challengerId));
        requestElt.appendChild(challengerElt);

        Node receiverElt = doc.createElement("receiverId");
        receiverElt.setTextContent(String.valueOf(receiverId));
        requestElt.appendChild(receiverElt);

        Node puzzleElt = doc.createElement("puzzleId");
        puzzleElt.setTextContent(String.valueOf(puzzleId));
        requestElt.appendChild(puzzleElt);

        Node puzzleNameElt = doc.createElement("puzzleName");
        puzzleNameElt.setTextContent("Test puzzlename");
        requestElt.appendChild(puzzleNameElt);

        Node dateElt = doc.createElement("requestDate");
        dateElt.setTextContent(new Date(new java.util.Date().getTime()).toString());
        requestElt.appendChild(dateElt);

        Node typeElt = doc.createElement("type");
        typeElt.setTextContent("speed");
        requestElt.appendChild(typeElt);

        Node challengeScoreElt = doc.createElement("challengeScore");
        challengeScoreElt.setTextContent(String.valueOf("550"));
        requestElt.appendChild(challengeScoreElt);

        Node receiverScoreElt = doc.createElement("receiverScore");
        receiverScoreElt.setTextContent("");
        requestElt.appendChild(receiverScoreElt);

        Node winnerElt = doc.createElement("winner");
        winnerElt.setTextContent("");
        requestElt.appendChild(winnerElt);
    }
    private Representation addUser1() throws ParserConfigurationException, SAXException, IOException {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        File userFile = helper.getUserFile();

        // Parsing the XML user file to create a DomRepresentation to send in the put method
        Document document = builder.parse(userFile);
        DomRepresentation rep = new DomRepresentation(MediaType.TEXT_XML, document);
        return userClient.put(rep);
    }
    private Representation addUser2() throws ParserConfigurationException, SAXException, IOException {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        File userFile = helper.getUserFile2();

        // Parsing the XML user file to create a DomRepresentation to send in the put method
        Document document = builder.parse(userFile);
        DomRepresentation rep = new DomRepresentation(MediaType.TEXT_XML, document);
        return userClient.put(rep);
    }
    private Representation addPuzzle() throws IOException, SAXException, ParserConfigurationException {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        File puzzleFile = helper.getPuzzleFile1();

        // Parsing the XML user file to create a DomRepresentation to send in the put method
        Document document = builder.parse(puzzleFile);
        DomRepresentation rep = new DomRepresentation(MediaType.TEXT_XML, document);
        return puzzleClient.put(rep);
    }

    @After
    public void deleteEverything() {
        puzzlesClient.delete();
        userClient.delete();
    }
}
